package ru.dbotthepony.mc.otm.client

import com.mojang.blaze3d.systems.RenderSystem
import net.minecraft.client.gui.components.Button
import net.minecraft.client.gui.screens.DeathScreen
import net.minecraft.client.gui.screens.InBedChatScreen
import net.minecraft.client.player.LocalPlayer
import net.minecraft.network.chat.Component
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.effect.MobEffects
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.entity.player.Player
import net.minecraftforge.client.event.RenderGuiEvent
import net.minecraftforge.client.event.RenderGuiOverlayEvent
import net.minecraftforge.client.event.ScreenEvent
import net.minecraftforge.client.gui.overlay.ForgeGui
import net.minecraftforge.client.gui.overlay.GuiOverlayManager
import net.minecraftforge.eventbus.api.EventPriority
import net.minecraftforge.eventbus.api.SubscribeEvent
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.MatteryPlayerCapability
import ru.dbotthepony.mc.otm.capability.matteryPlayer
import ru.dbotthepony.mc.otm.client.render.*
import ru.dbotthepony.mc.otm.core.RGBAColor
import ru.dbotthepony.mc.otm.core.ifPresentK
import java.util.*

object MatteryGUI {
	val PLAYER_GUI_LOCATION = ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/player_gui.png")

	private var knownButtonX = -1
	private var knownButtonY = -1
	private var knownButton: Button? = null
	private var knownButtonScreen: InBedChatScreen? = null

	private var lastState: MatteryPlayerCapability? = null

	private val button_shaker = Random()

	fun onScreenRender(event: ScreenEvent.Render.Pre) {
		if (knownButtonScreen != null && knownButton == null) {
			for (widget in knownButtonScreen!!.renderables) {
				if (widget is Button) {
					knownButtonX = widget.x
					knownButtonY = widget.y
					knownButton = widget
					break
				}
			}
			if (knownButton == null) {
				knownButtonScreen = null
			}
		}

		if (knownButtonScreen != null && knownButtonScreen !== minecraft.screen) {
			knownButtonX = -1
			knownButtonY = -1
			knownButton = null
			knownButtonScreen = null
		}

		if (knownButtonScreen == null || minecraft.player == null) {
			return
		}

		minecraft.player?.getCapability(MatteryCapability.MATTERY_PLAYER)?.ifPresentK {
			if (!it.willBecomeAndroid) {
				knownButton!!.x = knownButtonX
				knownButton!!.y = knownButtonY
				knownButtonX = -1
				knownButtonY = -1
				knownButton = null
				knownButtonScreen = null
				return@ifPresentK
			}

			val dispersion = (10.0 * Math.max(0, it.sleepTicks - 20) / (MatteryPlayerCapability.SLEEP_TICKS_LIMIT - 20)).toInt()

			knownButton!!.x =
				knownButtonX - dispersion / 2 + (button_shaker.nextDouble() * dispersion).toInt()
			knownButton!!.y =
				knownButtonY - dispersion / 2 + (button_shaker.nextDouble() * dispersion).toInt()
		}
	}

	fun onOpenGUIEvent(event: ScreenEvent.Opening) {
		knownButtonX = -1
		knownButtonY = -1
		knownButton = null
		knownButtonScreen = null

		if (minecraft.player == null) return

		val screen = event.screen

		if (screen is DeathScreen) {
			minecraft.player?.getCapability(MatteryCapability.MATTERY_PLAYER)?.ifPresentK {
				if (it.isAndroid) screen.title = TranslatableComponent("otm.death_reason")
			}
		}

		if (screen is InBedChatScreen) {
			knownButtonScreen = screen

			minecraft.player?.getCapability(MatteryCapability.MATTERY_PLAYER)?.ifPresentK {
				if (it.willBecomeAndroid) {
					knownButtonScreen = screen
				}
			}
		}
	}

	private val FOOD_LEVEL_ELEMENT by lazy {
		GuiOverlayManager.findOverlay(ResourceLocation("minecraft", "food_level"))
	}

	private val AIR_LEVEL_ELEMENT by lazy {
		GuiOverlayManager.findOverlay(ResourceLocation("minecraft", "air_level"))
	}

	var iteration = 0
	var showIterationUntil = 0L
	var showIterationUntilFade = 0L

	val deathLog = ArrayList<Pair<Int, Component>>()

	fun showIteration(event: RenderGuiEvent.Post) {
		if (minecraft.player?.matteryPlayer?.isAndroid != true) {
			return
		}

		val time = System.currentTimeMillis()

		if (time > showIterationUntilFade) {
			return
		}

		val stack = event.poseStack
		val window = event.window

		stack.pushPose()

		val negateScale = 1f / window.guiScale.toFloat()
		var modifyScale = 6f * negateScale
		val finalScale = modifyScale * window.guiScale.toFloat()

		stack.scale(modifyScale, modifyScale, modifyScale)

		var width = window.guiScaledWidth / modifyScale
		var height = window.guiScaledHeight / modifyScale

		var x = width / 2f
		var y = height / 2f

		val progress = if (time < showIterationUntil) 1f else 1f - (time - showIterationUntil).toFloat() / (showIterationUntilFade - showIterationUntil).toFloat()
		val scissorBase = (y - 12f) * finalScale
		val scissorHeight = (12f + (deathLog.size - 2f).coerceAtLeast(0f) * minecraft.font.lineHeight * modifyScale * 0.0875f) * finalScale

		pushScissorRect(0, (scissorBase + scissorHeight * (1f - progress)).toInt(), window.width, (scissorHeight * progress * 2f).toInt())

		setDrawColor(RGBAColor(1f, 1f, 1f, 0.4f))
		drawRect(stack, 0f, y - 12f, window.guiScaledWidth.toFloat(), 24f + (deathLog.size - 2f).coerceAtLeast(0f) * minecraft.font.lineHeight * modifyScale * 0.175f)

		val text = TranslatableComponent("otm.iteration", iteration)

		minecraft.font.drawAligned(stack, text, TextAlign.CENTER_CENTER, x + 1f, y + 1f, RGBAColor.BLACK)
		minecraft.font.drawAligned(stack, text, TextAlign.CENTER_CENTER, x, y, RGBAColor.WHITE)

		stack.scale(0.35f, 0.35f, 0.35f)

		modifyScale *= 0.35f

		width = window.guiScaledWidth / modifyScale
		height = window.guiScaledHeight / modifyScale

		x = width / 2f
		y = height / 2f + minecraft.font.lineHeight * finalScale * 0.35f

		var color = 0xFF

		for (i in deathLog.indices.reversed()) {
			val component = deathLog[i]
			minecraft.font.drawAligned(stack, component.second, TextAlign.CENTER_CENTER, x + 1f, y + 1f, RGBAColor.BLACK)
			minecraft.font.drawAligned(stack, component.second, TextAlign.CENTER_CENTER, x, y, RGBAColor(color, color, color))

			y += minecraft.font.lineHeight
			color = (color - 0x20).coerceAtLeast(0x0)
		}

		stack.popPose()

		popScissorRect()
	}

	fun onRenderGuiEvent(event: RenderGuiEvent.Post) {
		showIteration(event)
	}

	fun onLayerRenderEvent(event: RenderGuiOverlayEvent.Pre) {
		if (event.overlay != FOOD_LEVEL_ELEMENT && event.overlay != AIR_LEVEL_ELEMENT) {
			return
		}

		val ply: LocalPlayer = minecraft.player ?: return

		if (ply.vehicle is LivingEntity ||
			minecraft.options.hideGui ||
			minecraft.gameMode?.canHurtPlayer() == false ||
			minecraft.getCameraEntity() !is Player
		) {
			return
		}

		var android = ply.matteryPlayer

		if (!ply.isAlive && android == null) {
			android = lastState
		}

		val gui = minecraft.gui as? ForgeGui ?: return

		if (android != null && android.isAndroid) {
			event.isCanceled = true
			lastState = android

			if (event.overlay === AIR_LEVEL_ELEMENT) {
				return
			}

			val yOffset = if (ply.hasEffect(MobEffects.HUNGER)) 18 else 0
			var level: Float

			if (android.androidEnergy.maxBatteryLevel.isZero) {
				level = 0f
			} else {
				level = android.androidEnergy.batteryLevel.div(android.androidEnergy.maxBatteryLevel).toFloat()

				if (level >= 0.98f)
					level = 1f
			}

			gui.setupOverlayRenderState(true, false)
			RenderSystem.setShaderColor(1.0f, 1.0f, 1.0f, 1.0f)
			RenderSystem.setShaderTexture(0, PLAYER_GUI_LOCATION)

			val width = event.window.guiScaledWidth
			val height = event.window.guiScaledHeight
			val left = width / 2 + 10
			val top: Int = height - gui.rightHeight
			gui.rightHeight += 10

			// Stack, x, y, blitOffset?, (float) image_x, (float) image_y, rect_x, rect_y, total_image_width, total_image_height
			// Stack, x, y, image_x, image_y, rect_x, rect_y
			gui.blit(event.poseStack, left, top, 0, yOffset, 80, 9)
			val leftPadding = Math.ceil((level * 79f - 0.5f).toDouble()).toInt()

			gui.blit(event.poseStack, left + 79 - leftPadding, top, 79 - leftPadding, yOffset + 9, leftPadding, 9)
		}
	}
}
