package ru.dbotthepony.mc.otm.client.render

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.*
import com.mojang.math.Matrix4f
import net.minecraft.client.renderer.GameRenderer
import org.lwjgl.opengl.GL11
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.core.RGBAColor
import kotlin.math.acos
import kotlin.math.cos
import kotlin.math.pow
import kotlin.math.sin

private val identity = Matrix4f().also { it.setIdentity() }

var zLevel = 0f
var drawColor = RGBAColor(255, 255, 255, 255)

@JvmName("setDrawColor\$JVM")
fun setDrawColor(color: RGBAColor) {
	drawColor = color
}

// Regular functions
fun drawTexturedRect(
	matrix: Matrix4f,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	u0: Float = 0f,
	v0: Float = 0f,
	u1: Float = 1f,
	v1: Float = 1f
) {
	RenderSystem.setShader(GameRenderer::getPositionTexShader)
	RenderSystem.enableTexture()
	RenderSystem.enableBlend()
	RenderSystem.defaultBlendFunc()
	RenderSystem.depthFunc(GL11.GL_ALWAYS)

	val builder = tesselator.builder

	builder.begin(VertexFormat.Mode.QUADS, DefaultVertexFormat.POSITION_TEX)
	builder.vertex(matrix, x, y + height, zLevel).uv(u0, v1).endVertex()
	builder.vertex(matrix, x + width, y + height, zLevel).uv(u1, v1).endVertex()
	builder.vertex(matrix, x + width, y, zLevel).uv(u1, v0).endVertex()
	builder.vertex(matrix, x, y, zLevel).uv(u0, v0).endVertex()

	BufferUploader.drawWithShader(builder.end())
}

fun drawTexturedRect(
	matrix: Matrix4f,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	uv: IUVCoords
) = drawTexturedRect(matrix, x, y, width, height, uv.u0, uv.v0, uv.u1, uv.v1)

fun drawTexturedRect(
	stack: PoseStack,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	uv: IUVCoords
) = drawTexturedRect(stack.last().pose(), x, y, width, height, uv.u0, uv.v0, uv.u1, uv.v1)

fun drawTexturedRect(
	stack: PoseStack,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	u0: Float,
	v0: Float,
	u1: Float,
	v1: Float
) = drawTexturedRect(stack.last().pose(), x, y, width, height, u0, v0, u1, v1)

fun drawTexturedRect(
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	uv: IUVCoords
) = drawTexturedRect(identity, x, y, width, height, uv.u0, uv.v0, uv.u1, uv.v1)

fun drawTexturedRect(
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	u0: Float,
	v0: Float,
	u1: Float,
	v1: Float
) = drawTexturedRect(identity, x, y, width, height, u0, v0, u1, v1)

fun colorSphere(matrix: Matrix4f, radius: Float) {
	val fragments = 32
	RenderSystem.enableBlend()
	RenderSystem.defaultBlendFunc()

	val builder = tesselator.builder
	builder.begin(VertexFormat.Mode.QUADS, DefaultVertexFormat.POSITION_COLOR)

	val turnStep = Math.PI * (1 / fragments.toDouble()) * 2
	val stripeStep = Math.PI * (1 / fragments.toDouble())

	for (turn in 0 until fragments) {
		val turnPre = turnStep * turn
		val turnPost = turnStep * (turn + 1)

		for (sector in 0 until fragments) {
			val tiltPre = Math.PI / 2 - stripeStep * sector
			val tiltPost = Math.PI / 2 - stripeStep * (sector + 1)
			val xPre = sin(turnPre).toFloat() * radius
			val xPost = sin(turnPost).toFloat() * radius
			val zPre = cos(turnPre).toFloat() * radius
			val zPost = cos(turnPost).toFloat() * radius
			val yPre = (sin(tiltPre) + 0.5).toFloat() * radius
			val yPost = (sin(tiltPost) + 0.5).toFloat() * radius

			builder.vertex(matrix, xPre * cos(tiltPost).toFloat(), yPost, zPre * cos(tiltPost).toFloat())
				.color(drawColor)
				.endVertex()

			builder.vertex(matrix, xPost * cos(tiltPost).toFloat(), yPost, zPost * cos(tiltPost).toFloat())
				.color(drawColor)
				.endVertex()

			builder.vertex(matrix, xPost * cos(tiltPre).toFloat(), yPre, zPost * cos(tiltPre).toFloat())
				.color(drawColor)
				.endVertex()

			builder.vertex(matrix, xPre * cos(tiltPre).toFloat(), yPre, zPre * cos(tiltPre).toFloat())
				.color(drawColor)
				.endVertex()
		}
	}

	BufferUploader.drawWithShader(builder.end())
}

fun colorSphere(pose: PoseStack, radius: Float) = colorSphere(pose.last().pose(), radius)

fun drawTexturedRectAuto(
	stack: PoseStack,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	image_x: Float,
	image_y: Float,
	mapped_width: Float,
	mapped_height: Float
) = drawTexturedRect(
	stack,
	x,
	y,
	width,
	height,
	image_x / mapped_width,
	image_y / mapped_height,
	(image_x + width) / mapped_width,
	(image_y + height) / mapped_height
)

fun drawTexturedRectAuto(
	stack: PoseStack,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	image_x: Float,
	image_y: Float,
	mapped_width: Float,
	mapped_height: Float,
	order: UVWindingOrder
) = drawTexturedRect(
	stack,
	x,
	y,
	width,
	height,
	order.translate(
		image_x / mapped_width,
		image_y / mapped_height,
		(image_x + width) / mapped_width,
		(image_y + height) / mapped_height
	)
)

fun drawTexturedRectAuto(
	matrix: Matrix4f,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	image_x: Float,
	image_y: Float,
	mapped_width: Float,
	mapped_height: Float
) = drawTexturedRect(
	matrix,
	x,
	y,
	width,
	height,
	image_x / mapped_width,
	image_y / mapped_height,
	(image_x + width) / mapped_width,
	(image_y + height) / mapped_height
)

fun drawTexturedRectAuto(
	matrix: Matrix4f,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	image_x: Float,
	image_y: Float,
	mapped_width: Float,
	mapped_height: Float,
	order: UVWindingOrder
) = drawTexturedRect(
	matrix,
	x,
	y,
	width,
	height,
	order.translate(
		image_x / mapped_width,
		image_y / mapped_height,
		(image_x + width) / mapped_width,
		(image_y + height) / mapped_height
	)
)


fun drawTexturedRectAuto(
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	image_x: Float,
	image_y: Float,
	mapped_width: Float,
	mapped_height: Float
) = drawTexturedRect(
	x,
	y,
	width,
	height,
	image_x / mapped_width,
	image_y / mapped_height,
	(image_x + width) / mapped_width,
	(image_y + height) / mapped_height
)

fun drawTexturedRectAuto(
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	image_x: Float,
	image_y: Float,
	mapped_width: Float,
	mapped_height: Float,
	order: UVWindingOrder
) = drawTexturedRect(
	x,
	y,
	width,
	height,
	order.translate(
		image_x / mapped_width,
		image_y / mapped_height,
		(image_x + width) / mapped_width,
		(image_y + height) / mapped_height
	)
)

fun drawRect(
	matrix: Matrix4f,
	x: Float,
	y: Float,
	width: Float,
	height: Float
) {
	RenderSystem.disableTexture()
	RenderSystem.enableBlend()
	RenderSystem.defaultBlendFunc()
	RenderSystem.setShader(GameRenderer::getPositionColorShader)
	RenderSystem.depthFunc(GL11.GL_ALWAYS)

	val tess = tesselator
	val builder = tess.builder

	builder.begin(VertexFormat.Mode.QUADS, DefaultVertexFormat.POSITION_COLOR)

	builder.vertex(matrix, x, y + height, zLevel).color(drawColor).endVertex()
	builder.vertex(matrix, x + width, y + height, zLevel).color(drawColor).endVertex()
	builder.vertex(matrix, x + width, y, zLevel).color(drawColor).endVertex()
	builder.vertex(matrix, x, y, zLevel).color(drawColor).endVertex()

	tess.end()
	RenderSystem.enableTexture()
}

fun drawRect(
	pose: PoseStack,
	x: Float,
	y: Float,
	width: Float,
	height: Float
) = drawRect(pose.last().pose(), x, y, width, height)

fun drawRect(
	x: Float,
	y: Float,
	width: Float,
	height: Float
) = drawRect(identity, x, y, width, height)

fun drawLine(
	matrix: Matrix4f,
	startX: Float,
	startY: Float,
	endX: Float,
	endY: Float,
	width: Float
) {
	RenderSystem.disableTexture()
	RenderSystem.enableBlend()
	RenderSystem.defaultBlendFunc()
	RenderSystem.setShader(GameRenderer::getPositionColorShader)
	RenderSystem.depthFunc(GL11.GL_ALWAYS)

	val tess = Tesselator.getInstance()
	val builder = tess.builder

	builder.begin(VertexFormat.Mode.QUADS, DefaultVertexFormat.POSITION_COLOR)

	val length = ((startX - endX).pow(2f) + (startY - endY).pow(2f)).pow(0.5f)
	val angle = acos((endX - startX) / length)

	val cos = cos(angle)
	val sin = sin(angle)

	val y0 = -width

	val y1 = width

	val x2 = length
	val y2 = width

	val x3 = length
	val y3 = -width

	builder.vertex(matrix,
		startX - y0 * sin,
		startY + y0 * cos,
		zLevel).color(drawColor).endVertex()

	builder.vertex(matrix,
		startX - y1 * sin,
		startY + y1 * cos,
		zLevel).color(drawColor).endVertex()

	builder.vertex(matrix,
		startX + x2 * cos - y2 * sin,
		startY + x2 * sin + y2 * cos,
		zLevel).color(drawColor).endVertex()

	builder.vertex(matrix,
		startX + x3 * cos - y3 * sin,
		startY + x3 * sin + y3 * cos,
		zLevel).color(drawColor).endVertex()

	tess.end()
	RenderSystem.enableTexture()
}

fun drawLine(
	pose: PoseStack,
	startX: Float,
	startY: Float,
	endX: Float,
	endY: Float,
	width: Float
) = drawLine(pose.last().pose(), startX, startY, endX, endY, width)

fun drawLine(
	startX: Float,
	startY: Float,
	endX: Float,
	endY: Float,
	width: Float
) = drawLine(identity, startX, startY, endX, endY, width)

private data class ScissorRect(val x: Int, val y: Int, val width: Int, val height: Int)
private val scissorStack = ArrayDeque<ScissorRect>()

@Suppress("NAME_SHADOWING")
fun pushScissorRect(x: Int, y: Int, width: Int, height: Int) {
	var x = x
	var y = y
	var width = width
	var height = height

	val peek = scissorStack.lastOrNull()

	if (peek != null) {
		x = x.coerceAtLeast(peek.x)
		y = y.coerceAtLeast(peek.y)
		width = width.coerceAtMost(peek.width)
		height = height.coerceAtMost(peek.height)

		if (peek.x == x && peek.y == y && peek.width == width && peek.height == height) {
			scissorStack.add(peek)
			return
		}
	}

	scissorStack.add(ScissorRect(x, y, width, height))
	y = minecraft.window.height - y - height
	RenderSystem.enableScissor(x, y, width, height)
}

fun popScissorRect() {
	scissorStack.removeLast()

	val peek = scissorStack.lastOrNull()

	if (peek == null) {
		RenderSystem.disableScissor()
		return
	}

	val y = minecraft.window.height - peek.y - peek.height
	RenderSystem.enableScissor(peek.x, y, peek.width, peek.height)
}

