package ru.dbotthepony.mc.otm.tests

import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.core.formatReadableNumber
import ru.dbotthepony.mc.otm.core.formatSi
import java.math.BigInteger

object FormattingTests {
	@Test
	@DisplayName("BigInteger formatting as readable number")
	fun biginteger() {
		assertEquals("0", BigInteger("0").formatReadableNumber())
		assertEquals("45", BigInteger("45").formatReadableNumber())
		assertEquals("-45", BigInteger("-45").formatReadableNumber())
		assertEquals("0", BigInteger("-0").formatReadableNumber())

		assertEquals("100", BigInteger("100").formatReadableNumber())
		assertEquals("-100", BigInteger("-100").formatReadableNumber())
		assertEquals("999", BigInteger("999").formatReadableNumber())
		assertEquals("1 999", BigInteger("1999").formatReadableNumber())
		assertEquals("8 992", BigInteger("8992").formatReadableNumber())
		assertEquals("-8 992", BigInteger("-8992").formatReadableNumber())
		assertEquals("100 200", BigInteger("100200").formatReadableNumber())
		assertEquals("-100 200", BigInteger("-100200").formatReadableNumber())

		assertEquals("-1 100 200", BigInteger("-1100200").formatReadableNumber())
		assertEquals("1 100 200", BigInteger("1100200").formatReadableNumber())
		assertEquals("2 730 250 200", BigInteger("2730250200").formatReadableNumber())
		assertEquals("1 222 730 250 200", BigInteger("1222730250200").formatReadableNumber())
	}

	@Test
	@DisplayName("BigInteger formatting as si number")
	fun bigintegerSi() {
		assertEquals("0", BigInteger("0").formatSi())
		assertEquals("420", BigInteger("420").formatSi())
		assertEquals("-420", BigInteger("-420").formatSi())
		assertEquals("555", BigInteger("555").formatSi())
		assertEquals("55", BigInteger("55").formatSi())
		assertEquals("1.20k", BigInteger("1205").formatSi())
		assertEquals("-1.20k", BigInteger("-1205").formatSi())
		assertEquals("1.21k", BigInteger("1215").formatSi())
		assertEquals("4.50M", BigInteger("4501204").formatSi())
		assertEquals("4.00M", BigInteger("4000111").formatSi())
		assertEquals("4.0011M", BigInteger("4001111").formatSi(4))
		assertEquals("-4.0011M", BigInteger("-4001111").formatSi(4))
	}

	@Test
	@DisplayName("ImpreciseFraction formatting as si number")
	fun impreciseFractionSi() {
		assertEquals("0.00", 				ImpreciseFraction("0").formatSi(2))
		assertEquals("14.62", 				ImpreciseFraction("14.62").formatSi(2))
		assertEquals("1.00k", 				ImpreciseFraction("1000").formatSi(2))
		assertEquals("1.00k", 				ImpreciseFraction("1000.1").formatSi(2))
		assertEquals("1.00k", 				ImpreciseFraction("1004.2").formatSi(2))
		assertEquals("1.01k", 				ImpreciseFraction("1014.5").formatSi(2))
		assertEquals("1.014k", 				ImpreciseFraction("1014.5").formatSi(3))
		assertEquals("1.01k", 				ImpreciseFraction("1014.256").formatSi(2))
		assertEquals("12.73k", 				ImpreciseFraction("12734.256").formatSi(2))
		assertEquals("127.34k", 			ImpreciseFraction("127342.256").formatSi(2))
		assertEquals("1.27M", 				ImpreciseFraction("1273421.256").formatSi(2))
		assertEquals("1.273M", 				ImpreciseFraction("1273421.256").formatSi(3))
		assertEquals("1.2734M", 			ImpreciseFraction("1273421.256").formatSi(4))
	}
}
